/*
  Simple test-retest reliability, allowing for a single residual variance, then for
different residuals in the two trials arising from extra error in Trial 1 (e.g., due
to familiarization) or extra error in Trial 2 (e.g., due to individual responses to
an intervention).
  Modify the program as shown in the proc mixed step to play with the random effects.
Choose the random-effects model appropriate to your specification of extra error on 
Trial 1 or 2.
  Values shown could be 100*log transformed &dep; hence SDs and mean effects are
approximate percents; e.g., 5 back-transforms to a factor of exp(5/100)=1.051 or 5.1%.
Back-transformation is not performed in this program.
*/
%let dep=VO2max; *the dependent variable;
%let mean=380; *true grand mean of dependent, back transforms to exp(380/100)=44.7;
%let sd=10; *true between-subject SD;
%let tem=3; *typical (standard) error of measurement;
%let deltamean=2; *change in mean Trial 2 minus Trial 1;
%let extraerror1=0; *more error on Trial 1, e.g., due to familiarization;
%let extraerror2=0; *more error on Trial 2, e.g., due to individual responses;
%let ssize=50; *sample size;
%let decdep=1; *decimal places for simple stats of dependent variable;
%let deceff=1; *decimal places for effects;
%let tValue=3.5; *threshold for outliers with ~50 residuals. Use 4.0 for ~500, 4.5 for ~5000;
%let alpha=0.1; *0.1 gives 90% confidence limits;

*generate the data;
data dat1;
do ChildID=1 to &ssize;
  True&dep=&mean+&sd*rannor(0);
  Trial=1;
  Obsvd&dep=True&dep+&tem*rannor(0)+&extraerror1*rannor(0);
  output;
  Trial=2;
  Obsvd&dep=True&dep+&deltamean+&tem*rannor(0)+&extraerror2*rannor(0);
  output;
  end;

title "Raw data, first 10 observations";
proc print data=dat1(obs=10);
format True&dep Obsvd&dep 5.&decdep;
run;

title "Simple stats";
proc means maxdec=&decdep;
var True&dep Obsvd&dep;
class Trial;
run;

*generate means and SDs for plotting;
proc sort data=dat1;
by Trial;

proc means noprint data=dat1;
var Obsvd&dep;
by Trial;
output out=means mean=Mean std=SD;

data means1;
set means;
MeanPlus=Mean+SD;
MeanMinus=Mean-SD;

data datplot;
set dat1(in=a) means1;

ods graphics / reset width=8cm height=11cm imagemap attrpriority=none;
title "Line diagram of individual values and means";
proc sgplot data=datplot noautolegend;
*styleattrs
  datacolors=(lightskyblue pink red orange yellow lightgreen lightskyblue magenta lightgrey)
  datasymbols=(circlefilled trianglefilled triangledownfilled squarefilled diamondfilled starfilled HomeDownFilled)
  datacontrastcolors=(black)
  datalinepatterns=(solid dash dot);
*reg x=Age y=Obsvd&dep/nomarkers lineattrs=(thickness=1) group=Sex ;
series x=Trial y=Obsvd&dep/lineattrs=(pattern=solid color=lightgrey) group=ChildID;
series x=Trial y=Mean/lineattrs=(pattern=solid color=black);
scatter x=Trial y=Mean/errorbarattrs=(color=black) yerrorlower=MeanMinus yerrorupper=MeanPlus
  filledoutlinedmarkers markerfillattrs=(color=white) 
  markeroutlineattrs=(color=black) markerattrs=(size=12 symbol=circlefilled);
xaxis values=(1,2) offsetmin=0.1 offsetmax=0.1;
yaxis label="&dep" labelpos=top;
run;

*Set up dummy variables for estimating extra varianc e in Trial 1 or Trial 2;
data dat2;
set dat1;
XvarTrial1=0;
if Trial=1 then XvarTrial1=1;
XvarTrial2=0;
if Trial=2 then XvarTrial2=1;

*clear datasets from any previous proc mixed, to avoid confusion if next proc mixed fails;
data clev cov est pred solf solr lsm lsmdiff;

ods select none;
title "Reliability mixed model; Sample size=&ssize; True mean=&mean; True between SD=&sd";
title2 "Delta mean Trial2-1=&deltamean; Typical error=&tem; Extra error Trial1,2=&extraerror1,&extraerror2";
proc mixed data=dat2 covtest cl alpha=&alpha nobound;
class ChildID Trial;
model Obsvd&dep=Trial/noint ddfm=sat solution cl alpha=&alpha residual outp=pred alphap=&alpha;
*Try this simplest reliability model first:;
random Int/subject=ChildID s cl alpha=&alpha; *This estimates true differences between subjects, and...
  ...without a repeated statement, the fixed and random effects provide a simple reliability analysis;
*Then star it off and try these 3 lines, but it may not run properly or at all with a small sample size:;
*random Int/subject=ChildID s cl alpha=&alpha; *This estimates true differences between subjects;
*repeated/group=Trial; *This estimates different residuals in Trial 1 and 2;
*parms 100 1 1;
*Then star off the above and try this:;
*repeated Trial/subject=ChildID type=un; *This is equivalent to the previous random+repeated statements, 
  and it always works, but it doesn't estimate residual variances as such;
*Then star it off and unstar one or other (but not both) of the following two lines...;
*...to estimate one residual and extra variance on either Trial 1 or Trial 2;
*random Int XvarTrial1/subject=ChildID s cl alpha=&alpha; *extra error on Trial 1;
*random Int XvarTrial2/subject=ChildID s cl alpha=&alpha; *or extra error on Trial 2;
*parms 100 1 5; *include with either of the above;
*Then try this:;
*repeated Trial*ChildID; *this illustrates how you can specify an actual residual with a repeated statement;
*The above effectively makes observations for Trials 1 and 2 come from different subjects;
*Then try this:;
*random Trial*ChildID; *this illustrates how you should not specify the residual with a random statement;
*With the above, SAS still estimates a residual, but it has no standard error, so the random-effet model is wrong;
estimate "Trial 2-1" Trial -1 1/cl alpha=&alpha;
lsmeans Trial/diff cl alpha=&alpha;
ods output classlevels=clev;
ods output covparms=cov;
ods output solutionf=solf;
ods output solutionr=solr;
ods output estimates=est;
ods output lsmeans=lsm;
ods output diffs=lsmdiff;
run;
ods select all;

title3 "Class levels";
proc print data=clev;
run;

title3 "Covparms (random effects)";
title4 "Check for proper estimation, especially the Residual";
proc print data=cov;
run;

data cov1;
length Group $ 10; *in case there is a group= in the random effects;
set cov;
array a Estimate Lower Upper; 
do over a;
  a=sign(a)*sqrt(sign(a)*a); *to allow for negative variance, with the random statement and nobound;
  end;
if Covparm="Residual" then CLtd=sqrt(Upper/Lower); *Confidence Limits in times/divide format. This is for residuals and it is a good approximation;
else CLpmApprox=(Upper-Lower)/2; *Confidence Limits in +/- format. This is for the other random effects with random and nobound;
DegFree=2*ZValue**2; *Degrees of freedom;
run;

title3 "Covparms (random effects) as SD";
proc print data=cov1;
var  CovParm Group Estimate CLtd CLpmApprox Lower Upper Alpha DegFree;
format Estimate Lower Upper CLpmApprox 5.&deceff CLtd 5.2 DegFree 5.1; 
run;

title3 "Fixed-effects solution";
proc print data=solf;
run;

data est1;
set est;
CLpm=(Upper-Lower)/2;
if Estimate=0 then Estimate=.;

title3 "Fixed-effect estimates";
proc print data=est1;
var Label Estimate CLpm Lower Upper Alpha DF;
format Estimate CLpm Lower Upper 5.&deceff DF 5.0;
run;

data lsm1;
set lsm;
CLpm=(Upper-Lower)/2;

title3 "Least-squares means";
proc print data=lsm1;
var Effect Trial Estimate CLpm Lower Upper DF;
format Estimate CLpm Lower Upper 5.&deceff DF 5.0;
run;

data lsmdiff1;
set lsmdiff;
CLpm=(Upper-Lower)/2;

title3 "Differences in least-squares means";
proc print data=lsmdiff1;
var Effect Trial _Trial Estimate CLpm Lower Upper DF;
format Estimate CLpm Lower Upper 5.&deceff DF 5.0;
run;

title3 "Residuals vs predicteds";
ods graphics / reset width=12cm height=10cm imagemap attrpriority=none;
proc sgplot data=pred; *uniform=all;
styleattrs
  datacolors=(lightskyblue pink red orange yellow lightgreen lightskyblue magenta lightgrey)
  datasymbols=(circlefilled trianglefilled triangledownfilled squarefilled diamondfilled starfilled HomeDownFilled)
  datacontrastcolors=(black);
refline 0/axis=y lineattrs=(pattern=dot thickness=1 color=black);
scatter x=pred y=Resid/filledoutlinedmarkers markerattrs=(size=10) group=Trial;
run;
ods graphics / reset;

title3 "Outliers (Standardized residual >&tvalue)";
title4 "Pred is the value of the dependent predicted by the model";
data pred1;
set pred;
if abs(StudentResid)>&tvalue;

proc print data=pred1;
format Obsvd&dep Pred Resid 5.&decdep StudentResid 5.1;
run;

title3 "Random-effects solution, first 10 observations";
proc print data=solr(obs=10);
format Estimate StdErrPred Lower Upper 5.&deceff;
run;





