/*
  This program adapts 09 Controlled trial 3 trials.sas by using Trial 1 as a baseline
trial to create a new dataset consisting of changes from Trial 1 in Trials 2 and 3.
The advantage of using change scores is easy visualization and estimation of the 
modifying effect of a subject characteristic on individual responses to the treatment.
The subject characteristic is included as a fixed-effect covariate. 
  Here the baseline (Trial 1) value of the dependent variable is used as a modifying 
covariate. The disadvantage for this subject characteristic is that for noisy measures
(typical error SD not negligible compared with true between-subject SD), the modifying
effect of baseline is attenuated. Hence, the individual responses due to baseline are not
fully explained by baseline, so the estimate of any remaining individual responses
is contaminated by the baseline effect on individual responses. 
  The changes in Trials 2 and 3 are analyzed separately, "by Trial".
  I have not simulated a real modifying effect of baseline in the control group, 
but the artifactual modifying effect arising from regression to the mean will be
apparent in both groups, depending on the magnitude of the typical error relative to 
the true between-subject SD. The difference in the slopes estimates the real modifying
effect of baseline.
  Modify the program as shown in the Proc Mixed step to use the repeated statement.
  Values shown could be 100*log transformed &dep; hence SDs and mean effects are
approximate percents; e.g., 5 back-transforms to a factor of exp(5/100)=1.051 or 5.1%.
Back-transformation is not &deped in this program.
*/
*c=control, e=exptal;
%let dep=VO2max; *the dependent variable;
%let meanc=380; *true grand mean in the first trial, back transforms to exp(380/100)=44.7;
%let meane=380; *true grand mean exptal in the first trial, same as meanc, if assume kids well randomized to control and exptal;
%let sdc=10; *true between-subject SD;
%let sde=10; *true between-subject SD;
%let tem=3; *typical (standard) error of measurement;
*%let teme=3; *typical (standard) error of measurement, make it the same in both groups;
%let deltameanc2=1; *change in control mean Trial 2 minus Trial 1, e.g., placebo effect;
%let deltameane2=5; *change in exptal mean Trial 2 minus Trial 1;
%let deltameanc3=2; *change in control mean Trial 3 minus Trial 1, e.g., placebo effect;
%let deltameane3=8; *change in exptal mean Trial 3 minus Trial 1;
%let extraerror1=0; *more error on Trial 1, due to familiarization;
*%let extraerror1e=0; *more error on Trial 1, due to familiarization;
*%let extraerror2c=0; *extra error on Trial 2, due to individual responses;
*%let extraerror3c=0; *extra error on Trial 3, due to individual responses;
%let extraerror2e=2; *extra error on Trial 2, due to individual responses;
%let extraerror23e=3; *more error on Trial 2 and 3, due to correlated individual responses;
%let extraerror3e=2; *extra error on Trial 3, due to individual responses;
%let covarfactor=-0.3; *fraction or multiple of subject's true SD contributing to 
 correlated individual responses in exptal group;
%let ssizec=20; *sample size;
%let ssizee=20; *sample size;
%let decdep=1; *decimal places for simple stats of dependent variable;
%let deceff=1; *decimal places for SDs and effects;
%let tValue=3.5; *threshold for outliers with ~50 residuals. Use 4.0 for ~500, 4.5 for ~5000;
%let alpha=0.1; *0.1 gives 90% confidence limits;

*generate the data;
data dat1;
Treat="control"; 
do ChildID=1 to &ssizec;
  True&dep=&meanc+&sdc*rannor(0);
  Trial=1;
  Obsvd&dep=True&dep+&tem*rannor(0)+&extraerror1*rannor(0);
  output;
  Trial=2;
  Obsvd&dep=True&dep+&deltameanc2+&tem*rannor(0);
  output;
  Trial=3;
  Obsvd&dep=True&dep+&deltameanc3+&tem*rannor(0);
  output;
  end;
Treat="exptal";
do ChildID=&ssizec+1 to &ssizec+&ssizee;
  True&dep=&meane+&sde*rannor(0);
  indresp23=&extraerror23e*rannor(0)+&covarfactor*(True&dep-&meane);
  Trial=1;
  Obsvd&dep=True&dep+&tem*rannor(0)+&extraerror1*rannor(0);
  output;
  Trial=2;
  Obsvd&dep=True&dep+&deltameane2+&tem*rannor(0)+&extraerror2e*rannor(0)+indresp23;
  output;
  Trial=3;
  Obsvd&dep=True&dep+&deltameane3+&tem*rannor(0)+&extraerror3e*rannor(0)+indresp23;
  output;
  end;

*Run from here if using data generated by Program 09 to compare the two analyses;
*This proc sort is necessary only when using data generated by Program 09;
proc sort data=dat1;
by Treat ChildID Trial;

*generate change scores;
data dat2;
set dat1;
retain Baseline;
if Trial=1 then Baseline=Obsvd&dep;
Delta&dep=Obsvd&dep-Baseline;
*if Trial ne 1; *keep Trial 1 here for plotting the change scores;

title "Raw data, first 10 observations (which will miss the exptal)";
proc print data=dat2(obs=10);
format True&dep Obsvd&dep Delta&dep 5.&decdep;
run;

title "Simple stats";
proc means maxdec=&decdep;
var True&dep Obsvd&dep Delta&dep;
class Treat Trial;
run;

proc sort data=dat2;
by Trial Treat;

ods graphics / reset width=10cm height=10cm imagemap attrpriority=none;
title "Scatterplot of change scores";
proc sgplot data=dat2;
styleattrs
  datacolors=(lightskyblue pink red orange yellow lightgreen lightskyblue magenta lightgrey)
  datasymbols=(circlefilled trianglefilled triangledownfilled squarefilled diamondfilled starfilled HomeDownFilled)
  datacontrastcolors=(black)
  datalinepatterns=(dash solid dot);
reg x=Baseline y=Delta&dep/nomarkers lineattrs=(thickness=1) group=Treat name="lines";
scatter x=Baseline y=Delta&dep/filledoutlinedmarkers markerattrs=(size=12) group=Treat name="points";
yaxis label="Delta&dep" labelpos=top;
by Trial;
where Trial ne 1;
keylegend "points" "lines";
run;

*generate means and SDs for plotting;
proc sort data=dat2;
by Treat Trial;

proc means noprint data=dat2;
var Delta&dep;
by Treat Trial;
output out=means mean=Mean std=SD;

data means1;
set means;
MeanPlus=Mean+SD;
MeanMinus=Mean-SD;

data datplot;
set dat2(in=a) means1;
*trick to get colored plots for individual values;
if a then do;
  if Treat="control" then Delta&dep.con=Delta&dep;
  else Delta&dep.exp=Delta&dep;
  end;
run;

ods graphics / reset width=8cm height=10cm imagemap attrpriority=none;
title "Line diagram of individual values, and means and SDs";
proc sgplot data=datplot;
styleattrs
  datacolors=(lightskyblue pink red orange yellow lightgreen lightskyblue magenta lightgrey)
  datasymbols=(circlefilled trianglefilled triangledownfilled squarefilled diamondfilled starfilled HomeDownFilled)
  datacontrastcolors=(black)
  datalinepatterns=(solid dash dot);
*reg x=Age y=Obsvd&dep/nomarkers lineattrs=(thickness=1) group=Sex ;
scatter x=Trial y=Mean/filledoutlinedmarkers markeroutlineattrs=(color=black) markerattrs=(size=12) 
  group=Treat groupdisplay=cluster clusterwidth=0.15; *duplication of points to get legend;
series x=Trial y=Delta&dep.con/lineattrs=(pattern=solid color=lightskyblue) group=ChildID;
series x=Trial y=Delta&dep.exp/lineattrs=(pattern=solid color=pink) group=ChildID;
series x=Trial y=Mean/lineattrs=(color=black pattern=solid)
  group=Treat groupdisplay=cluster clusterwidth=0.15;
scatter x=Trial y=Mean/errorbarattrs=(color=black) yerrorlower=MeanMinus yerrorupper=MeanPlus
  filledoutlinedmarkers markeroutlineattrs=(color=black) markerattrs=(size=12) 
  group=Treat groupdisplay=cluster clusterwidth=0.15 name="points";
xaxis values=(1,2,3) offsetmin=0.1 offsetmax=0.1;
yaxis label="Delta&dep" labelpos=top;
legenditem type=line name="linec"/lineattrs=(color=lightskyblue) label="control"; 
legenditem type=line name="linee"/lineattrs=(color=pink) label="exptal";
keylegend "points" "linec" "linee";
run;

*Set up dummy variable for estimating individual responses in Trials 2 and 3 in the exptal group;
data dat3;
set dat2;
XvarExp=0; *individual responses in exptal group on Trials 2 or 3;
if Treat="exptal" then XvarExp=1;

proc standard data=dat3 out=dat4 mean=0;
var Baseline;

data dat5;
set dat4;
if Trial ne 1; *delete Trial 1 ready to do the mixed model;

proc sort;
by Trial;

*clear datasets from any previous proc mixed, to avoid confusion if next proc mixed fails;
data clev cov est pred solf solr lsm lsmdiff;

ods select none;
title "Controlled-trial via change scores. Control, Exptal: Sample size=&ssizec, &ssizee; True mean=&meanc, &meane; true between SD=&sdc, &sde";
title2 "Delta mean Trial2-1=&deltameanc2, &deltameane2; Delta mean Trial3-1=&deltameanc3, &deltameane3";
title3 "Typical error=&tem; Indiv resp expt Trial 2=sqrt(&extraerror2e^2+&extraerror23e^2); Indiv resp expt Trial 3=sqrt(&extraerror3e^2+&extraerror23e^2)";
proc mixed data=dat5 covtest cl alpha=&alpha nobound;
class ChildID Treat; *always make the order here the same in the model statement;
model Delta&dep=Treat Treat*Baseline/noint ddfm=sat solution cl alpha=&alpha residual outp=pred alphap=&alpha;
*model Obsvd&dep=Treat Treat*Baseline/noint ddfm=sat solution cl alpha=&alpha outp=pred alphap=&alpha;
*the above model with Obsvd&dep works, but the modifying effect of Baseline in each group is harder to interpret;
random XvarExp/subject=ChildID s cl alpha=&alpha;
parms 5 5;
*Star off the above and unstar the repeated below to allow for the most general covparms,
 but without estimation of individual responses. Disregard the analysis of the covparms;
*repeated/group=Treat; 
estimate "Change scores:";
estimate "control" Treat 1 0/cl alpha=&alpha;
estimate "exptal" Treat 0 1/cl alpha=&alpha;
estimate "exptal-control" Treat -1 1/cl alpha=&alpha;
estimate "";
estimate "Effect of baseline:";
estimate "control" Treat*Baseline 1 0/cl alpha=&alpha;
estimate "exptal" Treat*Baseline 0 1/cl alpha=&alpha;
estimate "exptal-control" Treat*Baseline -1 1/cl alpha=&alpha;
lsmeans Treat/diff cl alpha=&alpha;
ods output classlevels=clev;
ods output covparms=cov;
ods output solutionf=solf;
ods output solutionr=solr;
ods output estimates=est;
ods output lsmeans=lsm;
ods output diffs=lsmdiff;
by Trial;
run;
ods select all;

title4 "Class levels";
proc print data=clev;
run;

title4 "Covparms (random effects)";
title5 "Check for proper estimation, especially the Residual";
proc print data=cov;
by Trial;
run;

data cov1;
length CovParm Group $ 12;
set cov;
array a Estimate Lower Upper; 
do over a;
  a=sign(a)*sqrt(sign(a)*a); *to allow for negative variance, with the random statement and nobound;
  end;
if Covparm="Residual" then CLtd=sqrt(Upper/Lower); *Confidence Limits in times/divide format. This is for residuals and it is a good approximation;
else CLpmApprox=(Upper-Lower)/2; *Confidence Limits in +/- format. This is for the other random effects with random and nobound;
DegFree=2*ZValue**2; *Degrees of freedom;
run;

title4 "Covparms (random effects) as SD";
title5 "Indiv resp Trials 2,3 (IndResp2,IndResp3)=sqrt(&extraerror2e^2+&extraerror23e^2), sqrt(&extraerror3e^2+&extraerror23e^2)";
title6 "The expected residual SD is &tem*sqrt(2) for typical error << true between-subjectSD";
title7 "down to &tem for typical error >> true between-subjectSD";
proc print data=cov1;
var CovParm Group Estimate CLtd CLpmApprox Lower Upper Alpha DegFree;
format Estimate Lower Upper CLpmApprox 5.&deceff CLtd 5.2 DegFree 5.0; 
by Trial;
run;

data lsm1;
set lsm;
CLpm=(Upper-Lower)/2;

title4 "Least-squares means";
proc print data=lsm1;
var Effect Treat Estimate CLpm Lower Upper DF;
format Estimate CLpm Lower Upper 5.&deceff DF 5.0;
by Trial;
run;

data lsmdiff1;
set lsmdiff;
CLpm=(Upper-Lower)/2;

title4 "Differences in least-squares means";
proc print data=lsmdiff1;
var Effect Treat _Treat Estimate CLpm Lower Upper DF;
format Estimate CLpm Lower Upper 5.&deceff DF 5.0;
by Trial;
run;

data est1;
set est;
CLpm=(Upper-Lower)/2;
if Estimate=0 then Estimate=.;

title4 "Fixed-effect estimates";
title5 "True effect of baseline exptal minus control in Trials 2 and 3 both = &covarfactor";
proc print data=est1;
var Label Estimate CLpm Lower Upper Alpha DF;
format Estimate CLpm Lower Upper 5.&deceff DF 5.0;
by Trial;
run;

title4 "Fixed-effect estimates (2 decimal places)";
title5 "True effect of baseline exptal minus control in Trials 2 and 3 both = &covarfactor";
proc print data=est1;
var Label Estimate CLpm Lower Upper Alpha DF;
format Estimate CLpm Lower Upper 5.2 DF 5.0;
by Trial;
run;
/*
title4 "Fixed-effects solution";
title5 "True effect of baseline exptal minus control in Trials 2 and 3 both = &covarfactor";
proc print data=solf;
by Trial;
run;
*/

title4 "Residuals vs predicteds";
ods graphics / reset width=12cm height=10cm imagemap attrpriority=none;
proc sgplot data=pred; *uniform=all;
styleattrs
  datacolors=(lightskyblue pink red orange yellow lightgreen lightskyblue magenta lightgrey)
  datasymbols=(circlefilled trianglefilled triangledownfilled squarefilled diamondfilled starfilled HomeDownFilled)
  datacontrastcolors=(black);
refline 0/axis=y lineattrs=(pattern=dot thickness=1 color=black);
scatter x=pred y=Resid/filledoutlinedmarkers markerattrs=(size=10) group=Treat;
by Trial;
run;
ods graphics / reset;

data predplot;
set pred;
if Treat="control" then ResidCon=Resid;
if Treat="exptal" then ResidExp=Resid;

title4 "Residuals vs Baseline, to check for non-linearity";
ods graphics / reset width=12cm height=13cm imagemap attrpriority=none;
proc sgplot data=predplot; *uniform=all;
styleattrs
  datacolors=(lightskyblue pink red orange yellow lightgreen lightskyblue magenta lightgrey)
  datasymbols=(circlefilled trianglefilled triangledownfilled squarefilled diamondfilled starfilled HomeDownFilled)
  datacontrastcolors=(black)
  datalinepatterns=(solid dash dot);
refline 0/axis=y lineattrs=(pattern=dot thickness=1 color=black);
scatter x=Baseline y=Resid/filledoutlinedmarkers markerattrs=(size=10) group=Treat name="points";
reg x=Baseline y=ResidCon/degree=1 lineattrs=(thickness=2 color=blue pattern=solid) name="linec" legendlabel="cont linear" nomarkers;
reg x=Baseline y=ResidExp/degree=1 lineattrs=(thickness=2 color=red pattern=solid) name="linee" legendlabel="expt linear" nomarkers;
reg x=Baseline y=ResidCon/degree=2 lineattrs=(thickness=2 color=blue pattern=dash) name="quadc" legendlabel="cont quadratic" nomarkers;
reg x=Baseline y=ResidExp/degree=2 lineattrs=(thickness=2 color=red  pattern=dash) name="quade" legendlabel="expt quadratic" nomarkers;
keylegend "points" "linec" "linee" "quadc" "quade";
yaxis labelpos=top;
by Trial;
run;
ods graphics / reset;

title4 "Outliers (Standardized residual >&tvalue)";
title5 "Pred is the value of the dependent predicted by the model";
data pred2;
set pred;
if abs(StudentResid)>&tvalue;

proc print data=pred2;
format Obsvd&dep Pred Resid 5.&decdep StudentResid 5.1;
run;

title4 "Random-effects solution of individual responses, first 10 observations in each Trial";
title5 "Solution not printed when individual response SD is negative";
proc print data=solr(obs=10);
format Estimate StdErrPred Lower Upper 5.&deceff;
where Trial=2 and 
  Effect ne "Intercept" and Estimate ne 0 and StdErrPred; *suppress the irrelevant non-contributing values;
run;

proc print data=solr(obs=10);
format Estimate StdErrPred Lower Upper 5.&deceff;
where Trial=3 and 
  Effect ne "Intercept" and Estimate ne 0 and StdErrPred; *suppress the irrelevant non-contributing values;
run;






